@@ -3,13 +3,14 @@ use strict;
 use warnings;
 use 5.008001;
 use parent qw/Exporter/;
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 use HTML::TreeBuilder::XPath;
 use LWP::UserAgent;
 use HTML::Selector::XPath 0.06 qw/selector_to_xpath/;
 use Scalar::Util qw/blessed refaddr/;
 use HTML::Entities qw/encode_entities/;
 
+use Scalar::Util qw/ refaddr /;
 our @EXPORT = qw/wq/;
 
 our $RESPONSE;
@@ -175,9 +176,11 @@ sub as_html {
     my $self = shift;
 
     my @html = map {
-        ref $_ ? $_->as_HTML( q{&<>'"}, $self->{indent}, {} )
-               : $_ }
-        @{$self->{trees}};
+        ref $_ ? $_->isa('HTML::TreeBuilder::XPath::TextNode') 
+                        ? $_->getValue 
+                        : $_->as_HTML( q{&<>'"}, $self->{indent}, {} )
+               : $_ 
+    } @{$self->{trees}};
 
     return wantarray ? @html : $html[0];
 }
@@ -226,7 +229,10 @@ sub attr {
 
 sub tagname {
     my $self = shift;
-    my @retval = map { $_->tag(@_) } @{$self->{trees}};
+    my @retval = map { ref $_ eq 'HTML::TreeBuilder::XPath::TextNode' 
+                            ? '#text' 
+                            : $_->tag(@_) 
+                } @{$self->{trees}};
     return wantarray ? @retval : $retval[0];
 }
 
@@ -473,9 +479,17 @@ sub add {
             @{$_->{trees}}; 
         } map { (ref $self || $self)->new($_) } @stuff;                
     }
-    
-    push @{$self->{trees}}, @nodes;
-    $self;
+
+    my %ids = map { $self->_node_id($_) => 1 } @{ $self->{trees} };
+
+    ( ref $self )->new_from_element( [ 
+        @{$self->{trees}}, grep { ! $ids{ $self->_node_id($_) } } @nodes  
+    ], $self );
+}
+
+sub _node_id {
+    my( undef, $node ) = @_;
+    refaddr $node;
 }
 
 sub prev {
@@ -490,11 +504,44 @@ sub prev {
 sub next {
     my $self = shift;
 
-    my @new = map { $_->getNextSibling } @{ $self->{trees} };
+    my @new = grep { $_ } map { $_->getNextSibling } @{ $self->{trees} };
 
     return (ref $self || $self)->new_from_element(\@new, $self);
 }
 
+sub not {
+    my( $self, $selector ) = @_;
+
+    my $class = ref $self;
+
+    my $xpath = ref $selector ? $$selector : selector_to_xpath($selector);
+    $self->filter(sub { ! grep { $_->matches($xpath) } grep { ref $_ } $class->new($_)->{trees}[0] } );
+}
+
+sub and_back {
+    my $self = shift;
+    
+    $self->add( $self->end );
+}
+
+sub next_until {
+    my( $self, $selector ) = @_;
+
+    my $class = ref $self;
+    my $collection = $class->new_from_element([],$self);
+
+    my $next = $self->next->not($selector);
+    while( $next->size ) {
+       $collection = $collection->add($next);
+       $next = $next->next->not( $selector );
+    }
+
+    # hide the loop from the outside world
+    $collection->{before} = $self;
+
+    return $collection;
+}
+
 sub last_response {
     my ($class) = @_;
     return $RESPONSE;
@@ -607,7 +654,7 @@ Create new instance of Web::Query from file name.
 
 =head3 add
 
-Add elements to the set of matched elements.
+Returns a new object augmented with the new element(s).
 
 =over 4
 
@@ -617,7 +664,13 @@ An HTML fragment to add to the set of matched elements.
 
 =item add(@elements)
 
-One or more @elements to add to the set of matched elements.
+One or more @elements to add to the set of matched elements. 
+
+@elements that already are part of the set are not added a second time.
+
+    my $group = $wq->find('#foo');         # collection has 1 element
+    $group = $group->add( '#bar', $wq );   # 2 elements
+    $group->add( '#foo', $wq );            # still 2 elements
 
 =item add($wq)
 
@@ -679,6 +732,20 @@ Return the last matching element.
 
 This method constructs a new Web::Query object from the last matching element.
 
+=head3 not($selector)
+
+Return all the elements not matching the C<$selector>.
+
+    # $do_for_love will be every thing, except #that
+    my $do_for_love = $wq->find('thing')->not('#that');
+
+=head3 and_back
+
+Add the previous set of elements to the current one.
+
+    # get the h1 plus everything until the next h1
+    $wq->find('h1')->next_until('h1')->and_back;
+
 =head3 map
 
 Creates a new array with the results of calling a provided function on every element.
@@ -701,6 +768,10 @@ Get the next node of each element in the current set of matched elements.
 
    my $next = $q->next;
 
+=head3 next_until( $selector )
+
+Get all subsequent siblings, up to (but not including) the next node matched C<$selector>.
+
 =head2 MANIPULATION
 
 =head3 add_class
