@@ -6,7 +6,7 @@ use parent qw/Web::Query Exporter/;
 use HTML::TreeBuilder::LibXML;
 
 
-our $VERSION = "0.27";
+our $VERSION = "0.29";
 
 our @EXPORT = qw/wq/;
 
@@ -53,7 +53,7 @@ sub next {
     my $self = shift;
     my @new;
     for my $tree (@{$self->{trees}}) {
-        push @new, $tree->right;
+        push @new, grep { $_ } $tree->right;
     }
     return (ref $self || $self)->new_from_element(\@new, $self);
 }
@@ -66,6 +66,7 @@ sub tagname {
     return wantarray ? @retval : $retval[0];
 }
 
+sub _node_id { $_[1]{node}->unique_key }
 1;
 __END__
 
